/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */

-- Erstellen der XEvent-Sitzung "LanglaufendeAbfragen"
-- Protokolliert Abfragen mit einer Laufzeit grer 1 Sekunde


-- Ermitteln der Ereignisse mit "sql" und "completed"
SELECT	P.name As Paket, O.name As Ereignis, O.description As Beschreibung, 
		'ADD EVENT ' + p.name + '.' + O.name As Syntax 
FROM	sys.dm_xe_packages P INNER JOIN sys.dm_xe_objects O
		ON P.guid = O.package_guid
WHERE	object_type = 'event' AND O.name LIKE '%sql%completed%';

-- Ermitteln der Ereignisfelder zum Ereignis "sql_statement_completed"
SELECT	name, description, object_name, 
		CASE WHEN column_type = 'customizable' THEN 'Optionale Zusatzinformation' END As OptionaleSpalte
FROM	sys.dm_xe_object_columns 
WHERE	object_name = 'sql_statement_completed';

-- Ermitteln der Aktionen
SELECT	object_type, name, description 
FROM	sys.dm_xe_objects 
WHERE	object_type = 'action' 
		AND (name LIKE '%database%' OR name LIKE '%name%' OR name LIKE '%handle%');



-- Sitzung lschen, falls bereits vorhanden
IF EXISTS(SELECT * FROM sys.server_event_sessions WHERE name='LanglaufendeAbfragen')
BEGIN
	DROP EVENT SESSION LanglaufendeAbfragen ON SERVER;
END
GO
-- Sitzung anlegen
CREATE EVENT SESSION [LanglaufendeAbfragen] ON SERVER 
	ADD EVENT sqlserver.sp_statement_completed
	(	SET collect_statement=1
		ACTION (sqlserver.database_id, sqlserver.database_name,
				sqlserver.client_hostname, sqlserver.username,
				sqlserver.plan_handle)
		WHERE	duration >= 1000000 AND sqlserver.database_id = 12
	),
	ADD EVENT sqlserver.sql_statement_completed
	(	SET collect_statement=1
		ACTION (sqlserver.database_id, sqlserver.database_name,
				sqlserver.client_hostname, sqlserver.username,
				sqlserver.plan_handle)
		WHERE   duration >= 1000000 AND sqlserver.database_id = 12
	) 
	ADD TARGET package0.event_file (SET filename=N'D:\SNEK2\Ergebnisse\LanglaufendeAbfragen.xel')
-- Latenzzeit aus Demozwecken auf 10 Sekunden
WITH (MAX_DISPATCH_LATENCY=10 SECONDS);


-- Ausgabe der Konfiguration
SELECT	Sitzungen.name As Sitzung, Ereignisse.package As Paket, Ereignisse.name As Ereignis,
		Ereignisse.predicate As Prdikate, Aktionen.name As Aktionen, 
		Ziele.name As Ziele, Eigenschaften.value As Speicher
FROM	sys.server_event_sessions Sitzungen
		INNER JOIN sys.server_event_session_events Ereignisse 
				ON Sitzungen.event_session_id = Ereignisse.event_session_id
		INNER JOIN sys.server_event_session_actions Aktionen
				ON Sitzungen.event_session_id = Aktionen.event_session_id
		INNER JOIN sys.server_event_session_targets Ziele
				ON Sitzungen.event_session_id = Ziele.event_session_id
		INNER JOIN sys.server_event_session_fields Eigenschaften
				ON Sitzungen.event_session_id = Eigenschaften.event_session_id
			  AND Ziele.target_id = Eigenschaften.[object_id] 
WHERE	Sitzungen.name = 'LanglaufendeAbfragen'
ORDER BY Ereignis, Aktionen;


-- Sitzung starten
ALTER EVENT SESSION LanglaufendeAbfragen ON SERVER STATE=START;



-- Testen
USE SNEK
GO

WAITFOR DELAY '00:00:03'
GO

SELECT [Bestell-Nr], [Artikel-Nr], Anzahl, Einzelpreis, Rabatt FROM dbo.Bestelldetails WHERE Einzelpreis > 0;
SELECT [Bestell-Nr], [Artikel-Nr], Anzahl, Einzelpreis, Rabatt FROM dbo.Bestelldetails WHERE Einzelpreis > 1;
GO

EXEC dbo.pSelectTest;
GO


-- Sitzung stoppen
-- ALTER EVENT SESSION TriggerAusfuehrung ON SERVER STATE=STOP;


