/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */

-- Auswerten der XEvent-Sitzung "Trigger"
-- Quelle: Ereignnisdatei


-- Ermitteln vom XML-Dokument in der Ergebnisdatei
SELECT	CAST(event_data AS XML) Ergebnis
FROM	sys.fn_xe_file_target_read_file('D:\SNEK2\Ergebnisse\Trigger*.xel', NULL, NULL, NULL);
		

-- Auswerten des XML-Dokuments mittels XQuery
SELECT	Ergebnis, 
		xmlWert.value('(event/@name)[1]', 'nvarchar(512)') AS Ereignis,
		xmlWert.value('(event/data[@name="object_type"]/value)[1]', 'int') AS ObjektID,
		xmlWert.value('(event/data[@name="object_type"]/text)[1]', 'nvarchar(max)') AS ObjektTyp,
		xmlWert.value('(event/action[@name="database_name"]/value)[1]', 'nvarchar(256)') AS Datenbank
FROM
		(SELECT CAST(event_data AS XML) Ergebnis
		 FROM sys.fn_xe_file_target_read_file('D:\SNEK2\Ergebnisse\Trigger*.xel', NULL, NULL, NULL)) Ergebnisdaten
		 CROSS APPLY Ergebnis.nodes ('.') AS xmlErgebnis (xmlWert);



-- Tatschliche Auswertung der Ergebnisdateien Trigger*.xel
-- Die Daten werden mittels einer CTE ausgegeben, um die Daten der CTE mit denen der Tabelle sys.objects zu ergnzen. 
-- Mit der Funktion "Object_Name" auf der Spalte "parent_object_id" der Tabelle sys.objects wird der Name der Tabelle 
-- ermittelt, an der der Trigger definiert ist.

USE SNEK
GO

WITH cteErgebnis (Ereignis, Zeitpunkt, DatenbankID, Datenbank, ObjektTyp, ObjektID, DauerInSek, CpuInSek, 
				  UrsprungSqlAnweisung, AnzahlZeilen, SqlAnweisungTrigger, Zeilennummer, AnzahlZeilenTrigger,
				  Benutzer, Client, Ergebnis)
AS
(
SELECT	xmlWert.value('(event/@name)[1]', 'nvarchar(512)') AS Ereignis,
		DATEADD(hh, DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
				xmlWert.value('(event/@timestamp)[1]', 'datetime2')) AS Zeitpunkt,
		xmlWert.value('(event/action[@name="database_id"]/value)[1]','int') AS DatenbankID,
		xmlWert.value('(event/action[@name="database_name"]/value)[1]', 'nvarchar(256)') AS Datenbank,
		xmlWert.value('(event/data[@name="object_type"]/text)[1]', 'nvarchar(max)') AS ObjektTyp,
		xmlWert.value('(event/data[@name="object_id"]/value)[1]', 'int') As ObjektID,
		CAST((xmlWert.value('(event/data[@name="duration"]/value)[1]', 'bigint') / 1000000.0) As decimal(6,2)) AS DauerInSek,
		CAST((xmlWert.value('(event/data[@name="cpu_time"]/value)[1]', 'bigint') / 1000000.0) As decimal(6,2)) AS CpuInSek,
		xmlWert.value('(event/action[@name="sql_text"]/value)[1]', 'nvarchar(max)') AS UrsprungSqlAnweisung,
		xmlWert.value('(event/data[@name="row_count"]/value)[1]', 'bigint') AS AnzahlZeilen,
		xmlWert.value('(event/data[@name="statement"]/value)[1]', 'nvarchar(max)') AS SqlAnweisungTrigger,
		xmlWert.value('(event/data[@name="line_number"]/value)[1]', 'int') AS Zeilennummer,
		xmlWert.value('(event/data[@name="last_row_count"]/value)[1]', 'bigint') AS AnzahlZeilenTrigger, 
		xmlWert.value('(event/action[@name="username"]/value)[1]', 'nvarchar(256)') AS Benutzer,
		xmlWert.value('(event/action[@name="client_hostname"]/value)[1]', 'nvarchar(256)') AS Client,
		Ergebnis
FROM
		(SELECT CAST(event_data AS XML) Ergebnis
		 FROM sys.fn_xe_file_target_read_file('D:\SNEK2\Ergebnisse\Trigger*.xel', NULL, NULL, NULL)) Ergebnisdaten
		 CROSS APPLY Ergebnis.nodes ('.') AS xmlErgebnis (xmlWert)
)
SELECT	Ereignis, Zeitpunkt, DatenbankID, Datenbank, Object_Name(parent_object_id) As Tabelle, 
		ObjektTyp, name As Objekt, DauerInSek, CpuInSek, UrsprungSqlAnweisung, 
		SqlAnweisungTrigger, Zeilennummer, AnzahlZeilenTrigger,
		Benutzer, Client, Ergebnis
FROM	cteErgebnis E
		INNER JOIN sys.objects O ON E.ObjektID = O.object_id
WHERE	AnzahlZeilen > 0							-- nur die Anweisungen vom Trigger, die auch Daten gendert haben
		OR SqlAnweisungTrigger LIKE 'RAISERROR%'	-- oder die einen Fehler verursacht haben
ORDER BY Zeitpunkt;