/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */

-- Auswerten der XEvent-Sitzung "Fehler"
-- Quelle: Ereignnisdatei
-- Die Daten werden zunchst in eine Table-Variable geschrieben.
-- Anschlieend wird die Table-Variable mit zwei SQL-Anweisungen ausgewertet.

DECLARE @tabErgebnis TABLE
(
	ID int IDENTITY,
	Ereignis nvarchar(50) NOT NULL, 
	Zeitpunkt datetime2 NOT NULL,
	Datenbank nvarchar(128) NULL, 
	FehlerNr int NULL, 
	Schweregrad int NULL, 
	Systemfehler bit NULL, 
	InFehlerbehandlung bit NULL,
	Fehlermeldung nvarchar(max) NULL, 
	SqlAnweisung nvarchar(max) NULL,
	Benutzer nvarchar(128) NULL, 
	Client nvarchar(128) NULL, 
	Ergebnis xml NOT NULL
);
INSERT INTO @tabErgebnis (Ereignis, Zeitpunkt, Datenbank, FehlerNr, Schweregrad, Systemfehler, 
						  InFehlerbehandlung, Fehlermeldung, SqlAnweisung, Benutzer, Client, Ergebnis)
SELECT	
		xmlWert.value('(event/@name)[1]', 'nvarchar(512)') As Ereignis,
		DATEADD(hh, DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP),
			xmlWert.value('(event/@timestamp)[1]', 'datetime2')) As Zeitpunkt,
		xmlWert.value('(event/action[@name="database_name"]/value)[1]', 'nvarchar(256)') As Datenbank,
		xmlWert.value('(event/data[@name="error_number"]/value)[1]', 'int') As FehlerNr,
		xmlWert.value('(event/data[@name="severity"]/value)[1]', 'int') As Schweregrad,
		CASE WHEN xmlWert.value('(event/data[@name="user_defined"]/value)[1]', 'bit') = 0 
				THEN 1 ELSE 0 END As Benutzerdefiniert,
		CASE WHEN xmlWert.value('(event/data[@name="is_intercepted"]/value)[1]', 'bit')  = 1 
				THEN 1 ELSE 0 END As InFehlerbehandlung,
		xmlWert.value('(event/data[@name="message"]/value)[1]', 'nvarchar(max)') As Fehlermeldung,
		xmlWert.value('(event/action[@name="sql_text"]/value)[1]', 'nvarchar(max)') As SqlText,
		xmlWert.value('(event/action[@name="username"]/value)[1]', 'nvarchar(256)') AS Benutzer,
		xmlWert.value('(event/action[@name="client_hostname"]/value)[1]', 'nvarchar(256)') AS Client,
		Ergebnis
FROM	
		(SELECT CAST(event_data AS XML) Ergebnis
			FROM	sys.fn_xe_file_target_read_file('D:\SNEK2\Ergebnisse\Fehler*.xel', NULL, NULL, NULL)) Ergebnisdaten
			CROSS APPLY Ergebnis.nodes ('.') AS xmlErgebnis (xmlWert)


-- Detaillierte Auswertung inklusive Sql-Anweisung
SELECT	Ereignis, Zeitpunkt, Datenbank, FehlerNr, Schweregrad, 
		Systemfehler, Fehlermeldung, SqlAnweisung,
		Benutzer, Client, Ergebnis
FROM	@tabErgebnis E 
ORDER BY Schweregrad DESC


-- Gruppierte Auswertung mit Anzahl der eingetretenen Fehler
SELECT	Ereignis, COUNT(Ereignis) As AnzVorkommen, Datenbank, FehlerNr, 
		Schweregrad, Systemfehler, InFehlerbehandlung, Fehlermeldung 
FROM	@tabErgebnis E
GROUP BY Ereignis, Datenbank, FehlerNr, Schweregrad, Systemfehler, InFehlerbehandlung, Fehlermeldung 


-- Eintrag "Using 'dbghelp.dll' version '4.0.5'"
-- Zugriff auf das Ergebnis der XEvents verwendet SQL DUMPER.
-- Die Verwendung von SQL DUMPER wird im Fehlerprotokoll eingetragen.