/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */

-- Auswerten der XEvent-Sitzung "LanglaufendeAbfragen"
-- Quelle: Ringbuffer


-- Ermitteln vom XML-Dokument im Ringpuffer 
SELECT	CAST(target_data AS XML) As Ergebnis
FROM	sys.dm_xe_sessions AS s 
		INNER JOIN sys.dm_xe_session_targets AS t 
				ON t.event_session_address = s.address
WHERE	t.target_name = 'ring_buffer' AND s.name = 'AehnlicheSqlAnweisungen'
		

-- Tatschliche Auswertung des Ringpuffers
-- Die Daten werden zunchst in zwei Table-Variablen geschrieben.
-- Table-Variable "@tabRingpuffer" enthlt Daten zum Ringpuffer.
-- Table-Variable "@tabErgebnis" enthlt Daten zu den Ereignissen.


DECLARE @tabRingpuffer TABLE
(
	ID int IDENTITY,
	VerwendeterArbeitsspeicher int NULL,
	AnzahlEintragungen int NULL,
	AnzahlEntfernteEintragungen int NULL,
	GesamtanzahlEintragungen int NULL,
	Ergebnis xml NOT NULL
);
DECLARE @tabErgebnis TABLE
(
	ID int IDENTITY,
	Ereignis varchar(50) NULL,
	Zeitpunkt datetime2 NULL,
	SqlAnweisung nvarchar(max) NULL,
	Zeilennummer int NULL,
	DauerInSek bigint NULL,
	CpuInSek bigint NULL,
	AnzahlDatenstze int NULL,
	LogischeLesevorgnge int NULL,
	PhysikalischeLesevorgnge int NULL,
	Schreibvorgnge int NULL,
	Benutzer nvarchar(128) NULL,
	Client nvarchar(128) NULL,
	QueryHash bigint NULL, 
	SessionID int NULL
);

-- Informationen zum Ringpuffer speichern
INSERT INTO @tabRingpuffer (VerwendeterArbeitsspeicher, AnzahlEintragungen, AnzahlEntfernteEintragungen, 
							GesamtanzahlEintragungen, Ergebnis)
SELECT	
		xmlWert.value('(RingBufferTarget/@memoryUsed)[1]', 'bigint') AS VerwendeterArbeitsspeicher,
		xmlWert.value('(RingBufferTarget/@eventCount)[1]', 'bigint') AS AnzahlEintragungen,
		xmlWert.value('(RingBufferTarget/@droppedCount)[1]', 'bigint') AS AnzahlEntfernteEintragungen,
		xmlWert.value('(RingBufferTarget/@totalEventsProcessed)[1]', 'bigint') AS GesamtanzahlEintragungen,
		Ergebnis
FROM 
		(SELECT	CAST(target_data AS XML) As Ergebnis
		 FROM	sys.dm_xe_sessions AS s INNER JOIN sys.dm_xe_session_targets AS t 
				ON t.event_session_address = s.address
		 WHERE	t.target_name = 'ring_buffer' AND s.name = 'AehnlicheSqlAnweisungen') Ergebnisdaten
		CROSS APPLY Ergebnis.nodes ('.') AS xmlErgebnis (xmlWert);

-- Daten der protokollierten Ereignisse speichern
INSERT INTO @tabErgebnis (Ereignis, Zeitpunkt, SqlAnweisung, Zeilennummer, DauerInSek, CpuInSek, 
						  AnzahlDatenstze, LogischeLesevorgnge, PhysikalischeLesevorgnge, 
						  Schreibvorgnge, Benutzer, Client, QueryHash, SessionID)
SELECT 
		xmlWert.value('(@name)[1]', 'varchar(512)') AS Ereignis,
		DATEADD(hh, DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
				xmlWert.value('(@timestamp)[1]', 'datetime2')) AS Zeitpunkt,
		xmlWert.value('(data[@name="statement"]/value)[1]', 'nvarchar(max)') as SqlAnweisung,
		xmlWert.value('(data[@name="line_number"]/value)[1]', 'int') as Zeilennummer,
		CAST((xmlWert.value('(data[@name="duration"]/value)[1]', 'bigint') / 1000000.0) As decimal(6,2)) AS DauerInSek,
		CAST((xmlWert.value('(data[@name="cpu_time"]/value)[1]', 'bigint') / 1000000.0) As decimal(6,2)) AS CpuInSek,
		xmlWert.value('(data[@name="row_count"]/value)[1]', 'bigint') as AnzahlDatenstze,
		xmlWert.value('(data[@name="logical_reads"]/value)[1]', 'bigint') as LogischeLesevorgnge,
		xmlWert.value('(data[@name="physical_reads"]/value)[1]', 'bigint') as PhysikalischeLesevorgnge,
		xmlWert.value('(data[@name="writes"]/value)[1]', 'bigint') as Schreibvorgnge,
		xmlWert.value('(action[@name="username"]/value)[1]', 'nvarchar(256)') as Benutzer,
		xmlWert.value('(action[@name="client_hostname"]/value)[1]', 'nvarchar(256)') as Client,		
		xmlWert.value('(action[@name="query_hash"]/value)[1]', 'bigint') as QueryHash,
		xmlWert.value('(action[@name="session_id"]/value)[1]', 'int') as SessionID
FROM 
		(SELECT	CAST(target_data AS XML) As Ergebnis
		 FROM	sys.dm_xe_sessions AS s INNER JOIN sys.dm_xe_session_targets AS t 
				ON t.event_session_address = s.address
		 WHERE	t.target_name = 'ring_buffer' AND s.name = 'AehnlicheSqlAnweisungen') Ergebnisdaten
		CROSS APPLY Ergebnis.nodes ('//RingBufferTarget/event') AS xmlErgebnis (xmlWert);


-- Auswerten der Daten zum Ringpuffer 
SELECT	VerwendeterArbeitsspeicher, AnzahlEintragungen, 
		AnzahlEntfernteEintragungen, GesamtanzahlEintragungen, Ergebnis
FROM	@tabRingpuffer;

-- Auswerten der Daten zu den Ereignissen
-- Gruppierung nach QueryHash, um die hnlichen Anweisungen auszugeben
SELECT	Ereignis, COUNT(*) As AnzAusfhrungen,  
		MIN(Zeitpunkt) As minZeitpunkt, MAX(Zeitpunkt) As maxZeitpunkt,
		AVG(DauerInSek) As avgDauer, AVG(CpuInSek) As AvgCPU, avg(AnzahlDatenstze) As avgAnzahlDatenstze,
		E.QueryHash,
		(SELECT TOP 1 'wie - ' + SqlAnweisung FROM @tabErgebnis WHERE Zeitpunkt = Min(E.Zeitpunkt)) As SqlAnweisung
FROM	@tabErgebnis E
GROUP BY Ereignis, E.QueryHash
ORDER BY AnzAusfhrungen DESC;