/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */

-- Auswerten der XEvent-Sitzung "SperrenZaehlen"
-- Quelle: Ereigniszhler und Histogramm


-- Auswerten des Ziels "Ereigniszhler"
-- Liefert Informationen, wie oft das Ergeinis "lock_aquired" ausgelst wurde.
-- Anders ausgedrckt: Wie oft wurden Sperren angefordert?
SELECT 
		xmlWert.value('../@name[1]', 'nvarchar(512)') As Paket,
		xmlWert.value('@name[1]', 'varchar(512)') As Ereignis,
		xmlWert.value('@count[1]', 'bigint') As Anzahl,
		Ergebnis
FROM
		(SELECT CAST(target_data as XML) Ergebnis
		 FROM	sys.dm_xe_sessions AS Sitzung INNER JOIN sys.dm_xe_session_targets Ziel
				ON Sitzung.address = Ziel.event_session_address
		 WHERE	Sitzung.name = 'SperrenZaehlen'	AND Ziel.target_name = 'event_counter') as Ergebnisdaten
		CROSS APPLY Ergebnis.nodes('//CounterTarget/Packages/Package/Event') as xmlErgebnis (xmlWert);


-- Auswerten des Ziels "Ereigniszhler"
-- Liefert Informationen, wie oft beim Ergeinis "lock_aquired" die jeweiligen Objekttypen verwendet wurden..
-- Anders ausgedrckt: Wie oft wurden Sperren von welchem Typ angefordert?
-- Die Daten werden mit einer CTE ausgegeben, um das Ergbnis mit der DMF dm_xe_map_values zu ergnzen.
-- Die DMF dm_xe_map_values liefert die einzelnen Sperrtypen im Klartext.
WITH cteErgebnis (Anzahl, Sperrtyp, Sitzung, Ergebnis)
AS
(
	SELECT	xmlWert.value('(@count)[1]', 'bigint') AS Anzahl,
			xmlWert.value('(value)[1]', 'int') AS Sperrtyp,
			Sitzung, Ergebnis
	FROM	(SELECT Sitzung.[address] As Sitzung, CAST(target_data as XML) Ergebnis
			 FROM	sys.dm_xe_sessions AS Sitzung INNER JOIN sys.dm_xe_session_targets Ziel
					ON Sitzung.address = Ziel.event_session_address
			 WHERE	Sitzung.name = 'SperrenZaehlen' AND Ziel.target_name = 'histogram') as Ergebnisdaten
			CROSS APPLY Ergebnis.nodes('HistogramTarget/Slot') as xmlErgebnis (xmlWert)
)
SELECT	
		map_value As Sperrtyp, Anzahl, Ergebnis
FROM	
		cteErgebnis  
		INNER JOIN sys.dm_xe_session_events Ereignisse
				ON cteErgebnis.Sitzung = Ereignisse.event_session_address
		INNER JOIN sys.dm_xe_map_values Zuordnungen
				ON Ereignisse.event_package_guid = Zuordnungen.object_package_guid
			   AND cteErgebnis.Sperrtyp = Zuordnungen.map_key 
WHERE
		Ereignisse.event_name = 'lock_acquired' AND Zuordnungen.name = 'lock_mode';