/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */


-- Erstellen der XEvent-Sitzung "TimeOuts"
-- Protokolliert die Ereignisse zu denen eines der beiden im Ziel
-- definierten Ereignisse nicht eingetreten ist


-- Sitzung lschen, falls bereits vorhanden
IF EXISTS(SELECT * FROM sys.server_event_sessions WHERE name='TimeOuts')
BEGIN 
	DROP EVENT SESSION TimeOuts ON SERVER;
END
GO
-- Sitzung anlegen
CREATE EVENT SESSION TimeOuts ON SERVER
	ADD EVENT sqlserver.sql_statement_starting 
	(
		ACTION (sqlserver.client_hostname, sqlserver.username,
				sqlserver.database_name, sqlserver.database_id,
				sqlserver.tsql_stack, sqlserver.plan_handle, sqlserver.session_id)
		WHERE   sqlserver.database_id = 12
	),
	ADD EVENT sqlserver.sql_statement_completed 
    (
		ACTION (sqlserver.client_hostname, sqlserver.username,
				sqlserver.database_name, sqlserver.database_id,
				sqlserver.tsql_stack, sqlserver.plan_handle, sqlserver.session_id)
		WHERE   sqlserver.database_id = 12 
	)
ADD TARGET package0.pair_matching 
    (SET -- Start-Ereignis
		 begin_event = 'sqlserver.sql_statement_starting',
		 -- Ereignisfelder vom Start-Ereignis zum Herstellen der Verbindung
		 begin_matching_actions = 'sqlserver.session_id, sqlserver.tsql_stack',
		 -- End-Ereignis
		 end_event = 'sqlserver.sql_statement_completed',
		 -- Ereignisfelder vom End-Ereignis zum Herstellen der Verbindung
		 end_matching_actions = 'sqlserver.session_id, sqlserver.tsql_stack')
-- Latenzzeit aus Demozwecken auf 10 Sekunden und aktivieren der Kausalittsverfolgung 
WITH (MAX_DISPATCH_LATENCY=10 SECONDS, TRACK_CAUSALITY=ON);


-- Ausgabe der Konfiguration
SELECT	Sitzungen.name As Sitzung, Ereignisse.package As Paket, Ereignisse.name As Ereignis,
		Ereignisse.predicate As Prdikate, Aktionen.name As Aktionen, 
		Ziele.name As Ziele, Eigenschaften.value As Speicher
FROM	sys.server_event_sessions Sitzungen
		INNER JOIN sys.server_event_session_events Ereignisse 
				ON Sitzungen.event_session_id = Ereignisse.event_session_id
		INNER JOIN sys.server_event_session_actions Aktionen
				ON Sitzungen.event_session_id = Aktionen.event_session_id
		INNER JOIN sys.server_event_session_targets Ziele
				ON Sitzungen.event_session_id = Ziele.event_session_id
		INNER JOIN sys.server_event_session_fields Eigenschaften
				ON Sitzungen.event_session_id = Eigenschaften.event_session_id
			  AND Ziele.target_id = Eigenschaften.[object_id] 
WHERE	Sitzungen.name = 'Timeouts'
ORDER BY Ereignis, Aktionen;


-- Sitzung starten
ALTER EVENT SESSION TimeOuts ON SERVER STATE = START;



-- Testen
-- PassThrough-Abfrage in Access-Datenbank "SNEK2.accdb" ausfhren



-- Sitzung stoppen
-- ALTER EVENT SESSION TimeOuts ON SERVER STATE = STOP;
