/* -------------------------------------------------
Beispielskript zur SNEK 2 im April 2013 in Nrnberg
Autor: Bernd Jungbluth
Keine Gewhrleistung, keine Haftung
------------------------------------------------- */


-- Auswerten der XEvent-Sitzung "Deadlock"
-- Quelle: Ereignisdatei
-- Die Daten werden zunchst in eine Table-Variable geschrieben.
-- Anschlieend wird die Table-Variable mit vier SQL-Anweisungen ausgewertet.


DECLARE @Ergebnis TABLE
(
	Ereignis nvarchar(512),
	Zeitpunkt datetime2,
	DeadlockGraph xml,
	Opfer nvarchar(50),
	Prozess1 nvarchar(50),
	SqlAnweisung1 nvarchar(max),
	Client1 nvarchar(256),
	Benutzer1 nvarchar(256),
	SpID1 int,
	Prozess2 nvarchar(50),
	SqlAnweisung2 nvarchar(max),
	Client2 nvarchar(256),
	Benutzer2 nvarchar(256),
	SpID2 int,
	DatenbankID int,
	Objekt1 nvarchar(256),
	AktiverProcessObjekt1 nvarchar(50),
	SperrmodusAktiverProcessObjekt1 nvarchar(5),
	WartenderProcessObjekt1 nvarchar(50),
	SperrmodusProcessObjekt1 nvarchar(5),
	Objekt2 nvarchar(256),
	AktiverProcessObjekt2 nvarchar(50),
	SperrmodusAktiverProcessObjekt2 nvarchar(5),
	WartenderProcessObjekt2 nvarchar(50),
	SperrmodusProcessObjekt2 nvarchar(5),
	Ergebnis xml
);
INSERT INTO @Ergebnis
SELECT	xmlWert.value('(event/@name)[1]', 'nvarchar(512)') AS Ereignis,
		DATEADD(hh, DATEDIFF(hh, GETUTCDATE(), CURRENT_TIMESTAMP), 
		xmlWert.value('(event/@timestamp)[1]', 'datetime2')) AS Zeitpunkt,	
		xmlWert.query('//event/data/value/deadlock') As DeadlockGraph,
		xmlWert.value('(event/data/value/deadlock/victim-list/victimProcess/@id)[1]', 'nvarchar(50)') AS Opfer,		
		xmlWert.value('(event/data/value/deadlock/process-list/process/@id)[1]', 'nvarchar(50)') AS Prozess1,						
		xmlWert.value('(event/data/value/deadlock/process-list/process/inputbuf)[1]', 'nvarchar(max)') AS Anweisung1,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@hostname)[1]', 'nvarchar(256)') AS Client1,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@loginname)[1]', 'nvarchar(256)') AS Benutzer1,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@spid)[1]', 'int') AS SpID1,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@id)[2]', 'nvarchar(50)') AS Prozess2,						
		xmlWert.value('(event/data/value/deadlock/process-list/process/inputbuf)[2]', 'nvarchar(max)') AS Anweisung2,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@hostname)[2]', 'nvarchar(256)') AS Client2,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@loginname)[2]', 'nvarchar(256)') AS Benutzer2,
		xmlWert.value('(event/data/value/deadlock/process-list/process/@spid)[2]', 'int') AS SpID2,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/@dbid)[1]', 'int') AS DatenbankID,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/@objectname)[1]', 'nvarchar(256)') AS Objekt1,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/owner-list/owner/@id)[1]', 'nvarchar(50)') AS AktiverProcessObjekt1,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/owner-list/owner/@mode)[1]', 'nvarchar(5)') AS SperrmodusAktiverProcessObjekt1,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/waiter-list/waiter/@id)[1]', 'nvarchar(50)') AS WartenderProcessObjekt1,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/waiter-list/waiter/@mode)[1]', 'nvarchar(5)') AS SperrmodusProcessObjekt1,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/@objectname)[2]', 'nvarchar(max)') AS Objekt2,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/owner-list/owner/@id)[2]', 'nvarchar(50)') AS AktiverProcessObjekt2,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/owner-list/owner/@mode)[2]', 'nvarchar(5)') AS SperrmodusAktiverProcessObjekt2,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/waiter-list/waiter/@id)[2]', 'nvarchar(50)') AS WartenderProcessObjekt2,
		xmlWert.value('(event/data/value/deadlock/resource-list/keylock/waiter-list/waiter/@mode)[2]', 'nvarchar(5)') AS SperrmodusProcessObjekt2,
		Ergebnis 
FROM	
		(SELECT	CAST(event_data AS XML) Ergebnis
		FROM	sys.fn_xe_file_target_read_file('D:\SNEK2\Ergebnisse\Deadlocks*.xel', NULL, NULL, NULL)) Ergebnisdaten
		CROSS APPLY Ergebnis.nodes ('.') AS xmlErgebnis (xmlWert);



-- Detaillierte Auswertung 
SELECT	Ereignis, Zeitpunkt, Prozess1, Client1, Benutzer1, 
		Prozess2, Client2, Benutzer2, Opfer, Ergebnis
FROM	@Ergebnis;


-- Gegenberstellung der Daten zum Opfer und zum Gewinner
SELECT	'Opfer' As Opfer, Opfer As Prozess,
		CASE WHEN Opfer = Prozess1 THEN SpID1 ELSE SpID2 END As SPID,
		CASE WHEN Opfer = Prozess1 THEN Client1 ELSE Client2 END As Client,
		CASE WHEN Opfer = Prozess1 THEN Benutzer1 ELSE Benutzer2 END As Benutzer,
		CASE WHEN Opfer = Prozess1 THEN SqlAnweisung1 ELSE SqlAnweisung2 END As SqlAnweisung
FROM	@Ergebnis
UNION ALL
SELECT	'Gewinner' As Gewinner, 
		CASE WHEN Opfer = Prozess1 THEN Prozess2 ELSE Prozess1 END As Prozess,
		CASE WHEN Opfer != Prozess1 THEN SpID1 ELSE SpID2 END As SPID,
		CASE WHEN Opfer != Prozess1 THEN Client1 ELSE Client2 END As Client,
		CASE WHEN Opfer != Prozess1 THEN Benutzer1 ELSE Benutzer2 END As Benutzer,
		CASE WHEN Opfer != Prozess1 THEN SqlAnweisung1 ELSE SqlAnweisung2 END As SqlAnweisung
FROM	@Ergebnis;


-- Gegenberstellung der Daten zu den beteiligten Objekten 
SELECT	db_name(DatenbankID) As Datenbank,
		Objekt1 As Objekt, AktiverProcessObjekt1 As AktiverProcessObjekt, 
		SperrmodusAktiverProcessObjekt1 As SperrmodusAktiverProcessObjekt, 
		WartenderProcessObjekt1 As WartenderProcessObjekt, 
		SperrmodusProcessObjekt1 As SperrmodusProcessObjekt
FROM	@Ergebnis
UNION ALL
SELECT	db_name(DatenbankID) As Datenbank,
		Objekt2, AktiverProcessObjekt2, SperrmodusAktiverProcessObjekt2, 
		WartenderProcessObjekt2, SperrmodusProcessObjekt2
FROM	@Ergebnis;



-- Ausgabe des XML-Dokuments zur Erstellung des Deadlock-Graphs
-- Ergebnis per Klick ffnen und mit der Erweiterung .xdl speichern
-- Anschlieend die Datei per Doppelklick ffnen.
SELECT	DeadlockGraph FROM	@Ergebnis;